==> echosms-0.1.7.tar.gz: echosms-0.1.7/src/example_code.py <==
# %%
"""Examples of using the echoSMs code to estimate scatter from objects."""

import matplotlib.pyplot as plt
import numpy as np

from echosms import MSSModel, PSMSModel, DCMModel, ESModel, PTDWBAModel
from echosms import BenchmarkData
from echosms import ReferenceModels
from echosms import as_dataframe, as_dataarray

# Load the reference model defintiions
rm = ReferenceModels()
print('Available reference models are:\n')
print('\n'.join(rm.names()))

# Load the benchmark data (from Jech et al., 2015)
bm = BenchmarkData()
bmf = bm.freq_dataset
bmt = bm.angle_dataset


def plot_compare(f1, ts1, label1, f2, ts2, label2, title):
    """Plot together two ts results TS."""
    jech_index = np.mean(np.abs(np.array(ts1) - np.array(ts2)))
    # Plot the mss model and benchmark results
    fig, axs = plt.subplots(2, 1, sharex=True)
    axs[0].plot(f1/1e3, ts1, label=label1)
    axs[0].plot(f2/1e3, ts2, label=label2)
    axs[0].set_ylabel('TS re 1 m$^2$ [dB]')
    axs[0].legend(frameon=False, fontsize=6)

    # Plot difference between the two ts datasets
    axs[1].plot(f1/1e3, np.array(ts1)-np.array(ts2), color='black')
    axs[1].set_xlabel('Frequency [kHz]')
    axs[1].set_ylabel(r'$\Delta$ TS [dB]')
    axs[1].annotate(f'{jech_index:.2f} dB', (0.05, 0.80), xycoords='axes fraction',
                    backgroundcolor=[.8, .8, .8])
    plt.suptitle(title)
    plt.show()


# %% ###############################################################################################
# Run the benchmark models and compare to the frequency-varying benchmark results.

# This is the mapping between model name from ReferenceModels and the appropriate column of data in
# BenchMarkData and which model to use.
models = [('weakly scattering sphere', 'Sphere_WeaklyScattering'),
          ('fixed rigid sphere', 'Sphere_Rigid'),
          ('pressure release sphere', 'Sphere_PressureRelease'),
          ('gas filled sphere', 'Sphere_Gas'),
          ('spherical fluid shell with pressure release interior', 'ShellSphere_PressureRelease'),
          ('spherical fluid shell with gas interior', 'ShellSphere_Gas'),
          ('spherical fluid shell with weakly scattering interior',
           'ShellSphere_WeaklyScattering'),
          ('fixed rigid finite cylinder', 'Cylinder_Rigid'),
          ('pressure release finite cylinder', 'Cylinder_PressureRelease'),
          ('gas filled finite cylinder', 'Cylinder_Gas'),
          ('weakly scattering finite cylinder', 'Cylinder_WeaklyScattering'),
          # ('fixed rigid prolate spheroid', 'ProlateSpheroid_Rigid'),
          ('pressure release prolate spheroid', 'ProlateSpheroid_PressureRelease'),
          # ('gas filled prolate spheroid', 'ProlateSpheroid_Gas'),
          # ('weakly scattering prolate spheroid', 'ProlateSpheroid_WeaklyScattering'),
          ]

for name, bm_name in models:
    # Get the model parameters used in Jech et al. (2015) for a particular model.
    s = rm.specification(name)
    m = rm.parameters(name)

    # create the appropriate model instance
    match s['benchmark_model']:
        case 'mss':
            mod = MSSModel()
        case 'psms':
            mod = PSMSModel()
        case 'dcm':
            mod = DCMModel()
        case _:
            pass

    # Add frequencies that have finite TS values and incident angle to the model parameters
    m['f'] = bmf['Frequency_kHz'][~np.isnan(bmf[bm_name])]*1e3  # [Hz]
    m['theta'] = 90.0

    # and run these
    ts = mod.calculate_ts(m)

    # The finite benchmark TS values
    bm_ts = bmf[bm_name][~np.isnan(bmf[bm_name])]

    plot_compare(m['f'], ts, s['benchmark_model'], m['f'], bm_ts, 'Benchmark', name)

# %% ###############################################################################################
# Run the benchmark models and compare to the angle-varying benchmark results.
models = [('fixed rigid finite cylinder', 'Cylinder_Rigid'),
          ('pressure release finite cylinder', 'Cylinder_PressureRelease'),
          ('gas filled finite cylinder', 'Cylinder_Gas'),
          ('weakly scattering finite cylinder', 'Cylinder_WeaklyScattering')]

for name, bm_name in models:
    # Get the model parameters used in Jech et al. (2015) for a particular model.
    s = rm.specification(name)
    m = rm.parameters(name)

    # create the appropriate model instance
    match s['benchmark_model']:
        case 'mss':
            mod = MSSModel()
        case 'psms':
            mod = PSMSModel()
        case 'dcm':
            mod = DCMModel()
        case _:
            pass
    # Get the model parameters used in Jech et al. (2015) for a particular model.
    s = rm.specification(name)
    m = rm.parameters(name)

    # Add frequencies and angle to the model parameters
    m['f'] = 38000  # [Hz]
    m['theta'] = bmt['Angle_deg']

    # and run these
    ts = mod.calculate_ts(m)

    jech_index = np.mean(np.abs(ts - bmt[bm_name]))

    # Plot the mss model and benchmark results
    fig, axs = plt.subplots(2, 1, sharex=True)
    axs[0].plot(m['theta'], ts, label='echoSMs')
    axs[0].plot(bmt['Angle_deg'], bmt[bm_name], label='Benchmark')
    axs[0].set_ylabel('TS re 1 m$^2$ [dB]')
    axs[0].legend(frameon=False, fontsize=6)

    # Plot difference between benchmark values and newly calculated mss model values
    axs[1].plot(m['theta'], ts-bmt[bm_name], color='black')
    axs[1].set_xlabel('Angle (°)')
    axs[1].set_ylabel(r'$\Delta$ TS [dB]')
    axs[1].annotate(f'{jech_index:.2f} dB', (0.05, 0.80), xycoords='axes fraction',
                    backgroundcolor=[.8, .8, .8])
    plt.suptitle(name)
    plt.show()

# %% ###############################################################################################
# Use the ES model on a calibration sphere
name = 'WC38.1 calibration sphere'
p = rm.parameters(name)
p['f'] = np.arange(10, 100, 0.05) * 1e3  # [kHz]

es = ESModel()
ts = es.calculate_ts(p)

plt.plot(p['f']*1e-3, ts)
plt.xlabel('Freq [kHz]')
plt.ylabel('TS re 1 m$^2$ [dB] ')
plt.title(name)
plt.show()

# Can readily modify the parameters for a different sphere
p['a'] = 0.012/2
ts = es.calculate_ts(p)
plt.plot(p['f']*1e-3, ts)
plt.xlabel('Freq [kHz]')
plt.ylabel('TS re 1 m$^2$ [dB] ')
plt.title('WC12 calibration sphere')
plt.show()

# %% ###############################################################################################
# Some other ways to run models.

mss = MSSModel()

# Can add more variations to the model parameters
m = rm.parameters('weakly scattering sphere')
m['f'] = np.linspace(12, 200, num=800) * 1e3  # [Hz]
m['target_rho'] = np.arange(1020, 1030, 1)  # [kg/m^3]
m['theta'] = [0, 90.0, 180.0]
# can convert this to a dataframe
models_df = as_dataframe(m, mss.no_expand_parameters)
# could also make a DataFrame of parameters that are not just the combination of all input
# parameters. This offers a way to specify a more tailored set of model parameters.

print(f'Running {len(models_df)} models')
# and run. This will return a Series
ts = mss.calculate_ts(models_df, multiprocess=True)
models_df['ts'] = ts

# Alternatively, the ts results can be added to the dataframe that is passed in:
# ts = mss.calculate_ts(models_df, multiprocess=True, result_type='expand')

# plot some of the results
for rho in m['target_rho']:
    r = models_df.query('target_rho == @rho and theta==90')
    plt.plot(r['f']/1e3, r['ts'], label=f'{rho:.0f}')

plt.xlabel('Freq [kHz]')
plt.ylabel('TS re 1 m$^2$ [dB] ')
plt.legend(title='Density [kg m$^{-3}$]')
plt.show()

# %% ###############################################################################################
# Example of model parameters not from the benchmarks
params = {'medium_rho': [1000, 1250, 1500],
          'medium_c': np.arange(1400, 1600, 100),
          'f': np.linspace(12, 100, num=400) * 1000,
          'theta': np.arange(0, 180, 1),
          'a': 0.07,
          'boundary_type': 'fluid filled',
          'target_c': 1450,
          'target_rho': 1250}

mss = MSSModel()

# Instead of converting params to a dataframe, an xarray can be used.
params_xa = as_dataarray(params, mss.no_expand_parameters)

# how many models runs would that be?
print(f'Running {np.prod(params_xa.shape)} models!')

# and is called the same way as for the dataframe. Use multiprocessing for this one as there
# are a lot of models to run.
mss.calculate_ts(params_xa, multiprocess=True)

# Xarray selections and dimenions names can then be used
plt.plot(params_xa.f, params_xa.sel(theta=90, medium_rho=1000, medium_c=1600))

# %% ###############################################################################################
# Example of PT-DWBA model

# The benchmark sphere model
m = rm.parameters('weakly scattering sphere')

# make a 3d matrix of 0's and 1's and set to 1 for the sphere
# and 0 for not the sphere
m['voxel_size'] = (0.0001, 0.0001, 0.0001)  # [m]
x = np.arange(-m['a'], m['a'], m['voxel_size'][0])
(X, Y, Z) = np.meshgrid(x, x, x)

m['volume'] = (np.sqrt(X**2 + Y**2 + Z**2) <= m['a']).astype(int)
m['theta'] = 90
m['phi'] = 0
m['rho'] = [m['medium_rho'], m['target_rho']]
m['c'] = [m['medium_c'], m['target_c']]
m['f'] = bmf['Frequency_kHz']*1e3
# remove unneeded parameters
m = {k: v for k, v in m.items()
     if k not in ['boundary_type', 'a', 'medium_rho', 'medium_c', 'target_rho', 'target_c']}

pt = PTDWBAModel()
dwba_ts = pt.calculate_ts(m)

plot_compare(m['f'], dwba_ts, 'PT-DWBA',
             m['f'], bmf['Sphere_WeaklyScattering'], 'Benchmark',
             'weakly scattering sphere')

# So, this PT_DWBA on a weakly scattering sphere is also different from the benchmark
# TS values. Hmmmm. Now look at the difference between the PT-DWBA and MSS model runs...

mss = MSSModel()
mm = rm.parameters('weakly scattering sphere')
mm['f'] = bmf['Frequency_kHz']*1e3
mm['theta'] = 90.0

mss_ts = mss.calculate_ts(mm)

plot_compare(mm['f'], dwba_ts, 'PT-DWBA',
             mm['f'], mss_ts, 'MSS',
             'weakly scattering sphere')

########################################################
# And then the same thing, but for the prolate spheroid
m = rm.parameters('weakly scattering prolate spheroid')
m['voxel_size'] = (0.0001, 0.0001, 0.0001)  # [m]
x = np.arange(-m['b'], m['b'], m['voxel_size'][0])
y = np.arange(-m['b'], m['b'], m['voxel_size'][0])
z = np.arange(-m['a'], m['a'], m['voxel_size'][0])
(X, Y, Z) = np.meshgrid(x, y, z)

m['volume'] = ((X**2 + Y**2)/m['b']**2 + Z**2/m['a']**2) <= 1
m['theta'] = 0
m['phi'] = 0
m['rho'] = [m['medium_rho'], m['target_rho']]
m['c'] = [m['medium_c'], m['target_c']]
# remove unneeded parameters
m = {k: v for k, v in m.items()
     if k not in ['boundary_type', 'a', 'b', 'medium_rho', 'medium_c', 'target_rho', 'target_c']}

dwba_ts = []
for f in freqs:
    print(f/1e3)
    m['f'] = f
    # PTDWBAModel() doesn't yet support calling via the calculate_ts() call.
    dwba_ts.append(pt.calculate_ts_single(**m))

plot_compare(freqs, dwba_ts, 'PT-DWBA',
             freqs, bmf['ProlateSpheroid_WeaklyScattering'], 'Benchmark',
             'weakly scattering prolate spheroid')

==> echosms-0.1.8.tar.gz: echosms-0.1.8/src/example_code.py <==
# %%
"""Examples of using the echoSMs code to estimate scatter from objects."""

import matplotlib.pyplot as plt
import numpy as np

from echosms import MSSModel, PSMSModel, DCMModel, ESModel, PTDWBAModel
from echosms import BenchmarkData
from echosms import ReferenceModels
from echosms import as_dataframe, as_dataarray

# Load the reference model defintiions
rm = ReferenceModels()
print('Available reference models are:\n')
print('\n'.join(rm.names()))

# Load the benchmark data (from Jech et al., 2015)
bm = BenchmarkData()
bmf = bm.freq_dataset
bmt = bm.angle_dataset


def plot_compare(f1, ts1, label1, f2, ts2, label2, title):
    """Plot together two ts results TS."""
    jech_index = np.nanmean(np.abs(np.array(ts1) - np.array(ts2)))
    # Plot the mss model and benchmark results
    fig, axs = plt.subplots(2, 1, sharex=True)
    axs[0].plot(f1/1e3, ts1, label=label1)
    axs[0].plot(f2/1e3, ts2, label=label2)
    axs[0].set_ylabel('TS re 1 m$^2$ [dB]')
    axs[0].legend(frameon=False, fontsize=6)

    # Plot difference between the two ts datasets
    axs[1].plot(f1/1e3, np.array(ts1)-np.array(ts2), color='black')
    axs[1].set_xlabel('Frequency [kHz]')
    axs[1].set_ylabel(r'$\Delta$ TS [dB]')
    axs[1].annotate(f'{jech_index:.2f} dB', (0.05, 0.80), xycoords='axes fraction',
                    backgroundcolor=[.8, .8, .8])
    plt.suptitle(title)
    plt.show()


# %% ###############################################################################################
# Run the benchmark models and compare to the frequency-varying benchmark results.

# This is the mapping between model name from ReferenceModels and the appropriate column of data in
# BenchMarkData and which model to use.
models = [('weakly scattering sphere', 'Sphere_WeaklyScattering'),
          ('fixed rigid sphere', 'Sphere_Rigid'),
          ('pressure release sphere', 'Sphere_PressureRelease'),
          ('gas filled sphere', 'Sphere_Gas'),
          ('spherical fluid shell with pressure release interior', 'ShellSphere_PressureRelease'),
          ('spherical fluid shell with gas interior', 'ShellSphere_Gas'),
          ('spherical fluid shell with weakly scattering interior',
           'ShellSphere_WeaklyScattering'),
          ('fixed rigid finite cylinder', 'Cylinder_Rigid'),
          ('pressure release finite cylinder', 'Cylinder_PressureRelease'),
          ('gas filled finite cylinder', 'Cylinder_Gas'),
          ('weakly scattering finite cylinder', 'Cylinder_WeaklyScattering'),
          # ('fixed rigid prolate spheroid', 'ProlateSpheroid_Rigid'),
          # ('pressure release prolate spheroid', 'ProlateSpheroid_PressureRelease'),
          # ('gas filled prolate spheroid', 'ProlateSpheroid_Gas'),
          # ('weakly scattering prolate spheroid', 'ProlateSpheroid_WeaklyScattering'),
          ]

for name, bm_name in models:
    # Get the model parameters used in Jech et al. (2015) for a particular model.
    s = rm.specification(name)
    m = rm.parameters(name)

    # create the appropriate model instance
    match s['benchmark_model']:
        case 'mss':
            mod = MSSModel()
        case 'psms':
            mod = PSMSModel()
        case 'dcm':
            mod = DCMModel()
        case _:
            pass

    # Add frequencies that have finite TS values and incident angle to the model parameters
    m['f'] = bmf['Frequency_kHz'][~np.isnan(bmf[bm_name])]*1e3  # [Hz]
    m['theta'] = 90.0

    # and run these
    ts = mod.calculate_ts(m)

    # The finite benchmark TS values
    bm_ts = bmf[bm_name][~np.isnan(bmf[bm_name])]

    plot_compare(m['f'], ts, s['benchmark_model'], m['f'], bm_ts, 'Benchmark', name)

# %% ###############################################################################################
# Run the benchmark models and compare to the angle-varying benchmark results.
models = [('fixed rigid finite cylinder', 'Cylinder_Rigid'),
          ('pressure release finite cylinder', 'Cylinder_PressureRelease'),
          ('gas filled finite cylinder', 'Cylinder_Gas'),
          ('weakly scattering finite cylinder', 'Cylinder_WeaklyScattering')]

for name, bm_name in models:
    # Get the model parameters used in Jech et al. (2015) for a particular model.
    s = rm.specification(name)
    m = rm.parameters(name)

    # create the appropriate model instance
    match s['benchmark_model']:
        case 'mss':
            mod = MSSModel()
        case 'psms':
            mod = PSMSModel()
        case 'dcm':
            mod = DCMModel()
        case _:
            pass
    # Get the model parameters used in Jech et al. (2015) for a particular model.
    s = rm.specification(name)
    m = rm.parameters(name)

    # Add frequencies and angle to the model parameters
    m['f'] = 38000  # [Hz]
    m['theta'] = bmt['Angle_deg']

    # and run these
    ts = mod.calculate_ts(m)

    jech_index = np.mean(np.abs(ts - bmt[bm_name]))

    # Plot the mss model and benchmark results
    fig, axs = plt.subplots(2, 1, sharex=True)
    axs[0].plot(m['theta'], ts, label='echoSMs')
    axs[0].plot(bmt['Angle_deg'], bmt[bm_name], label='Benchmark')
    axs[0].set_ylabel('TS re 1 m$^2$ [dB]')
    axs[0].legend(frameon=False, fontsize=6)

    # Plot difference between benchmark values and newly calculated mss model values
    axs[1].plot(m['theta'], ts-bmt[bm_name], color='black')
    axs[1].set_xlabel('Angle (°)')
    axs[1].set_ylabel(r'$\Delta$ TS [dB]')
    axs[1].annotate(f'{jech_index:.2f} dB', (0.05, 0.80), xycoords='axes fraction',
                    backgroundcolor=[.8, .8, .8])
    plt.suptitle(name)
    plt.show()

# %% ###############################################################################################
# Use the ES model on a calibration sphere
name = 'WC38.1 calibration sphere'
p = rm.parameters(name)
p['f'] = np.arange(10, 100, 0.05) * 1e3  # [kHz]

es = ESModel()
ts = es.calculate_ts(p)

plt.plot(p['f']*1e-3, ts)
plt.xlabel('Freq [kHz]')
plt.ylabel('TS re 1 m$^2$ [dB] ')
plt.title(name)
plt.show()

# Can readily modify the parameters for a different sphere
p['a'] = 0.012/2
ts = es.calculate_ts(p)
plt.plot(p['f']*1e-3, ts)
plt.xlabel('Freq [kHz]')
plt.ylabel('TS re 1 m$^2$ [dB] ')
plt.title('WC12 calibration sphere')
plt.show()

# %% ###############################################################################################
# Some other ways to run models.

mss = MSSModel()

# Can add more variations to the model parameters
m = rm.parameters('weakly scattering sphere')
m['f'] = np.linspace(12, 200, num=800) * 1e3  # [Hz]
m['target_rho'] = np.arange(1020, 1030, 1)  # [kg/m^3]
m['theta'] = [0, 90.0, 180.0]
# can convert this to a dataframe
models_df = as_dataframe(m, mss.no_expand_parameters)
# could also make a DataFrame of parameters that are not just the combination of all input
# parameters. This offers a way to specify a more tailored set of model parameters.

print(f'Running {len(models_df)} models')
# and run. This will return a Series
ts = mss.calculate_ts(models_df, multiprocess=True)
models_df['ts'] = ts

# Alternatively, the ts results can be added to the dataframe that is passed in:
# ts = mss.calculate_ts(models_df, multiprocess=True, result_type='expand')

# plot some of the results
for rho in m['target_rho']:
    r = models_df.query('target_rho == @rho and theta==90')
    plt.plot(r['f']/1e3, r['ts'], label=f'{rho:.0f}')

plt.xlabel('Freq [kHz]')
plt.ylabel('TS re 1 m$^2$ [dB] ')
plt.legend(title='Density [kg m$^{-3}$]')
plt.show()

# %% ###############################################################################################
# Example of model parameters not from the benchmarks
params = {'medium_rho': [1000, 1250, 1500],
          'medium_c': np.arange(1400, 1600, 100),
          'f': np.linspace(12, 100, num=400) * 1000,
          'theta': np.arange(0, 180, 1),
          'a': 0.07,
          'boundary_type': 'fluid filled',
          'target_c': 1450,
          'target_rho': 1250}

mss = MSSModel()

# Instead of converting params to a dataframe, an xarray can be used.
params_xa = as_dataarray(params, mss.no_expand_parameters)

# how many models runs would that be?
print(f'Running {np.prod(params_xa.shape)} models!')

# and is called the same way as for the dataframe. Use multiprocessing for this one as there
# are a lot of models to run.
mss.calculate_ts(params_xa, multiprocess=True)

# Xarray selections and dimenions names can then be used
plt.plot(params_xa.f, params_xa.sel(theta=90, medium_rho=1000, medium_c=1600))

# %% ###############################################################################################
# Example of PT-DWBA model

# The benchmark sphere model
m = rm.parameters('weakly scattering sphere')

# make a 3d matrix of 0's and 1's and set to 1 for the sphere
# and 0 for not the sphere
m['voxel_size'] = (0.0001, 0.0001, 0.0001)  # [m]
x = np.arange(-m['a'], m['a'], m['voxel_size'][0])
(X, Y, Z) = np.meshgrid(x, x, x)

m['volume'] = (np.sqrt(X**2 + Y**2 + Z**2) <= m['a']).astype(int)
m['theta'] = 90
m['phi'] = 0
m['rho'] = [m['medium_rho'], m['target_rho']]
m['c'] = [m['medium_c'], m['target_c']]
m['f'] = bmf['Frequency_kHz']*1e3
# remove unneeded parameters
m = {k: v for k, v in m.items()
     if k not in ['boundary_type', 'a', 'medium_rho', 'medium_c', 'target_rho', 'target_c']}

pt = PTDWBAModel()
dwba_ts = pt.calculate_ts(m)

plot_compare(m['f'], dwba_ts, 'PT-DWBA',
             m['f'], bmf['Sphere_WeaklyScattering'], 'Benchmark',
             'weakly scattering sphere')

# So, this PT_DWBA on a weakly scattering sphere is also different from the benchmark
# TS values. Hmmmm. Now look at the difference between the PT-DWBA and MSS model runs...

mss = MSSModel()
mm = rm.parameters('weakly scattering sphere')
mm['f'] = bmf['Frequency_kHz']*1e3
mm['theta'] = 90.0

mss_ts = mss.calculate_ts(mm)

plot_compare(mm['f'], dwba_ts, 'PT-DWBA',
             mm['f'], mss_ts, 'MSS',
             'weakly scattering sphere')

########################################################
# And then the same thing, but for the prolate spheroid
m = rm.parameters('weakly scattering prolate spheroid')
m['voxel_size'] = (0.0001, 0.0001, 0.0001)  # [m]
x = np.arange(-m['b'], m['b'], m['voxel_size'][0])
y = np.arange(-m['b'], m['b'], m['voxel_size'][0])
z = np.arange(-m['a'], m['a'], m['voxel_size'][0])
(X, Y, Z) = np.meshgrid(x, y, z)

m['volume'] = ((X**2 + Y**2)/m['b']**2 + Z**2/m['a']**2) <= 1
m['theta'] = 0
m['phi'] = 0
m['rho'] = [m['medium_rho'], m['target_rho']]
m['c'] = [m['medium_c'], m['target_c']]
m['f'] = bmf['Frequency_kHz']*1e3
# remove unneeded parameters
m = {k: v for k, v in m.items()
     if k not in ['boundary_type', 'a', 'b', 'medium_rho', 'medium_c', 'target_rho', 'target_c']}

pt = PTDWBAModel()
dwba_ts = pt.calculate_ts(m, multiprocess=True)

plot_compare(m['f'], dwba_ts, 'PT-DWBA',
             m['f'], bmf['ProlateSpheroid_WeaklyScattering'], 'Benchmark',
             'weakly scattering prolate spheroid')
